// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.28.1
// 	protoc        v3.12.4
// source: domaininfo.proto

package domaininfo

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type SecLevel int32

const (
	// Does not do TLS.
	SecLevel_PLAIN SecLevel = 0
	// TLS client connection (no certificate validation).
	SecLevel_TLS_CLIENT SecLevel = 1
	// TLS, but with invalid certificates.
	SecLevel_TLS_INSECURE SecLevel = 2
	// TLS, with valid certificates.
	SecLevel_TLS_SECURE SecLevel = 3
)

// Enum value maps for SecLevel.
var (
	SecLevel_name = map[int32]string{
		0: "PLAIN",
		1: "TLS_CLIENT",
		2: "TLS_INSECURE",
		3: "TLS_SECURE",
	}
	SecLevel_value = map[string]int32{
		"PLAIN":        0,
		"TLS_CLIENT":   1,
		"TLS_INSECURE": 2,
		"TLS_SECURE":   3,
	}
)

func (x SecLevel) Enum() *SecLevel {
	p := new(SecLevel)
	*p = x
	return p
}

func (x SecLevel) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (SecLevel) Descriptor() protoreflect.EnumDescriptor {
	return file_domaininfo_proto_enumTypes[0].Descriptor()
}

func (SecLevel) Type() protoreflect.EnumType {
	return &file_domaininfo_proto_enumTypes[0]
}

func (x SecLevel) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use SecLevel.Descriptor instead.
func (SecLevel) EnumDescriptor() ([]byte, []int) {
	return file_domaininfo_proto_rawDescGZIP(), []int{0}
}

type Domain struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// Security level for mail coming from this domain (they send to us).
	IncomingSecLevel SecLevel `protobuf:"varint,2,opt,name=incoming_sec_level,json=incomingSecLevel,proto3,enum=domaininfo.SecLevel" json:"incoming_sec_level,omitempty"`
	// Security level for mail going to this domain (we send to them).
	OutgoingSecLevel SecLevel `protobuf:"varint,3,opt,name=outgoing_sec_level,json=outgoingSecLevel,proto3,enum=domaininfo.SecLevel" json:"outgoing_sec_level,omitempty"`
}

func (x *Domain) Reset() {
	*x = Domain{}
	if protoimpl.UnsafeEnabled {
		mi := &file_domaininfo_proto_msgTypes[0]
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		ms.StoreMessageInfo(mi)
	}
}

func (x *Domain) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Domain) ProtoMessage() {}

func (x *Domain) ProtoReflect() protoreflect.Message {
	mi := &file_domaininfo_proto_msgTypes[0]
	if protoimpl.UnsafeEnabled && x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Domain.ProtoReflect.Descriptor instead.
func (*Domain) Descriptor() ([]byte, []int) {
	return file_domaininfo_proto_rawDescGZIP(), []int{0}
}

func (x *Domain) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *Domain) GetIncomingSecLevel() SecLevel {
	if x != nil {
		return x.IncomingSecLevel
	}
	return SecLevel_PLAIN
}

func (x *Domain) GetOutgoingSecLevel() SecLevel {
	if x != nil {
		return x.OutgoingSecLevel
	}
	return SecLevel_PLAIN
}

var File_domaininfo_proto protoreflect.FileDescriptor

var file_domaininfo_proto_rawDesc = []byte{
	0x0a, 0x10, 0x64, 0x6f, 0x6d, 0x61, 0x69, 0x6e, 0x69, 0x6e, 0x66, 0x6f, 0x2e, 0x70, 0x72, 0x6f,
	0x74, 0x6f, 0x12, 0x0a, 0x64, 0x6f, 0x6d, 0x61, 0x69, 0x6e, 0x69, 0x6e, 0x66, 0x6f, 0x22, 0xa4,
	0x01, 0x0a, 0x06, 0x44, 0x6f, 0x6d, 0x61, 0x69, 0x6e, 0x12, 0x12, 0x0a, 0x04, 0x6e, 0x61, 0x6d,
	0x65, 0x18, 0x01, 0x20, 0x01, 0x28, 0x09, 0x52, 0x04, 0x6e, 0x61, 0x6d, 0x65, 0x12, 0x42, 0x0a,
	0x12, 0x69, 0x6e, 0x63, 0x6f, 0x6d, 0x69, 0x6e, 0x67, 0x5f, 0x73, 0x65, 0x63, 0x5f, 0x6c, 0x65,
	0x76, 0x65, 0x6c, 0x18, 0x02, 0x20, 0x01, 0x28, 0x0e, 0x32, 0x14, 0x2e, 0x64, 0x6f, 0x6d, 0x61,
	0x69, 0x6e, 0x69, 0x6e, 0x66, 0x6f, 0x2e, 0x53, 0x65, 0x63, 0x4c, 0x65, 0x76, 0x65, 0x6c, 0x52,
	0x10, 0x69, 0x6e, 0x63, 0x6f, 0x6d, 0x69, 0x6e, 0x67, 0x53, 0x65, 0x63, 0x4c, 0x65, 0x76, 0x65,
	0x6c, 0x12, 0x42, 0x0a, 0x12, 0x6f, 0x75, 0x74, 0x67, 0x6f, 0x69, 0x6e, 0x67, 0x5f, 0x73, 0x65,
	0x63, 0x5f, 0x6c, 0x65, 0x76, 0x65, 0x6c, 0x18, 0x03, 0x20, 0x01, 0x28, 0x0e, 0x32, 0x14, 0x2e,
	0x64, 0x6f, 0x6d, 0x61, 0x69, 0x6e, 0x69, 0x6e, 0x66, 0x6f, 0x2e, 0x53, 0x65, 0x63, 0x4c, 0x65,
	0x76, 0x65, 0x6c, 0x52, 0x10, 0x6f, 0x75, 0x74, 0x67, 0x6f, 0x69, 0x6e, 0x67, 0x53, 0x65, 0x63,
	0x4c, 0x65, 0x76, 0x65, 0x6c, 0x2a, 0x47, 0x0a, 0x08, 0x53, 0x65, 0x63, 0x4c, 0x65, 0x76, 0x65,
	0x6c, 0x12, 0x09, 0x0a, 0x05, 0x50, 0x4c, 0x41, 0x49, 0x4e, 0x10, 0x00, 0x12, 0x0e, 0x0a, 0x0a,
	0x54, 0x4c, 0x53, 0x5f, 0x43, 0x4c, 0x49, 0x45, 0x4e, 0x54, 0x10, 0x01, 0x12, 0x10, 0x0a, 0x0c,
	0x54, 0x4c, 0x53, 0x5f, 0x49, 0x4e, 0x53, 0x45, 0x43, 0x55, 0x52, 0x45, 0x10, 0x02, 0x12, 0x0e,
	0x0a, 0x0a, 0x54, 0x4c, 0x53, 0x5f, 0x53, 0x45, 0x43, 0x55, 0x52, 0x45, 0x10, 0x03, 0x42, 0x30,
	0x5a, 0x2e, 0x62, 0x6c, 0x69, 0x74, 0x69, 0x72, 0x69, 0x2e, 0x63, 0x6f, 0x6d, 0x2e, 0x61, 0x72,
	0x2f, 0x67, 0x6f, 0x2f, 0x63, 0x68, 0x61, 0x73, 0x71, 0x75, 0x69, 0x64, 0x2f, 0x69, 0x6e, 0x74,
	0x65, 0x72, 0x6e, 0x61, 0x6c, 0x2f, 0x64, 0x6f, 0x6d, 0x61, 0x69, 0x6e, 0x69, 0x6e, 0x66, 0x6f,
	0x62, 0x06, 0x70, 0x72, 0x6f, 0x74, 0x6f, 0x33,
}

var (
	file_domaininfo_proto_rawDescOnce sync.Once
	file_domaininfo_proto_rawDescData = file_domaininfo_proto_rawDesc
)

func file_domaininfo_proto_rawDescGZIP() []byte {
	file_domaininfo_proto_rawDescOnce.Do(func() {
		file_domaininfo_proto_rawDescData = protoimpl.X.CompressGZIP(file_domaininfo_proto_rawDescData)
	})
	return file_domaininfo_proto_rawDescData
}

var file_domaininfo_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_domaininfo_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_domaininfo_proto_goTypes = []interface{}{
	(SecLevel)(0),  // 0: domaininfo.SecLevel
	(*Domain)(nil), // 1: domaininfo.Domain
}
var file_domaininfo_proto_depIdxs = []int32{
	0, // 0: domaininfo.Domain.incoming_sec_level:type_name -> domaininfo.SecLevel
	0, // 1: domaininfo.Domain.outgoing_sec_level:type_name -> domaininfo.SecLevel
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_domaininfo_proto_init() }
func file_domaininfo_proto_init() {
	if File_domaininfo_proto != nil {
		return
	}
	if !protoimpl.UnsafeEnabled {
		file_domaininfo_proto_msgTypes[0].Exporter = func(v interface{}, i int) interface{} {
			switch v := v.(*Domain); i {
			case 0:
				return &v.state
			case 1:
				return &v.sizeCache
			case 2:
				return &v.unknownFields
			default:
				return nil
			}
		}
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: file_domaininfo_proto_rawDesc,
			NumEnums:      1,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_domaininfo_proto_goTypes,
		DependencyIndexes: file_domaininfo_proto_depIdxs,
		EnumInfos:         file_domaininfo_proto_enumTypes,
		MessageInfos:      file_domaininfo_proto_msgTypes,
	}.Build()
	File_domaininfo_proto = out.File
	file_domaininfo_proto_rawDesc = nil
	file_domaininfo_proto_goTypes = nil
	file_domaininfo_proto_depIdxs = nil
}
